import 'package:flutter/material.dart';

abstract class ILoginViewModel extends ChangeNotifier {
  bool get isLoading;
  String? get errorMessage;
  String? get emailErrorMessage;
  String? get passwordErrorMessage;
  void emailChanged(String newEmail);
  void passwordChanged(String newPassword);
  void loginUser();
}

class LoginScreen extends StatelessWidget {
  final ILoginViewModel _viewModel;
  const LoginScreen(this._viewModel, {Key? key}) : super(key: key);
  // final ILoginViewModel _viewModel;

  _loggedInButtonPressed() {
    _viewModel.loginUser();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(),
      body: Padding(
        padding: const EdgeInsets.all(8.0),
        child: AnimatedBuilder(
            animation: _viewModel,
            builder: (context, child) {
              final errorMessage = _viewModel.errorMessage;
              return Builder(builder: (context) {
                return Column(
                  children: [
                    TextFormField(
                      onChanged: _viewModel.emailChanged,
                      keyboardType: TextInputType.emailAddress,
                      autofillHints: const [
                        AutofillHints.username,
                        AutofillHints.email
                      ],
                      decoration: InputDecoration(
                          labelText: "Email",
                          errorText: _viewModel.emailErrorMessage),
                    ),
                    TextFormField(
                      onChanged: _viewModel.passwordChanged,
                      obscureText: true,
                      autofillHints: const [AutofillHints.password],
                      decoration: InputDecoration(
                          labelText: "Password",
                          errorText: _viewModel.passwordErrorMessage),
                    ),
                    const SizedBox(height: 8),
                    if (_viewModel.isLoading)
                      const Center(child: CircularProgressIndicator())
                    else
                      Center(
                        child: ElevatedButton(
                            onPressed: _loggedInButtonPressed,
                            child: const Text("Sign in")),
                      ),
                    if (errorMessage != null) Text(errorMessage)
                  ],
                );
              });
            }),
      ),
    );
  }
}
