import 'package:flutter_application_1/data/model/user.dart';
import 'package:flutter_application_1/ui%20/%20screens/login_screen.dart';

abstract class ILoginUseCases {
  Future<User?> checkUserCredentials(String username, String password);
}

class LoginViewModel extends ILoginViewModel {
  final ILoginUseCases _useCases;
  bool _isLoading = false;
  String? _emailErrorMessage;
  String? _passwordErrorMessage;
  String? _errorMessage;

  String? _email;
  String? _password;

  LoginViewModel(this._useCases);

  @override
  bool get isLoading => _isLoading;
  @override
  String? get errorMessage => _errorMessage;
  @override
  String? get emailErrorMessage => _emailErrorMessage;
  @override
  String? get passwordErrorMessage => _passwordErrorMessage;

  bool get _minimalInputValid =>
      _email != null &&
      _password != null &&
      _emailErrorMessage == null &&
      _passwordErrorMessage == null;

  @override
  void emailChanged(String newEmail) {
    _email = newEmail;
    _emailErrorMessage = null;
  }

  @override
  void passwordChanged(String newPassword) {
    _password = newPassword;
    _passwordErrorMessage = null;
  }

  @override
  void loginUser() async {
    final email = _email;
    final password = _password;
    if (email != null && password != null) {
      if (email.contains("@") == false) {
        _emailErrorMessage = "Email invalide!";
        notifyListeners();
      }
      if (password.isEmpty) {
        _passwordErrorMessage = "Password invalide!";
        notifyListeners();
      }
      if (_minimalInputValid) {
        _isLoading = true;
        notifyListeners();
        final user = await _useCases.checkUserCredentials(email, password);
        _isLoading = false;
        _errorMessage =
            user != null ? "Bienvenue ${user.email}" : "Compte non retrouvé?!";
        notifyListeners();
      }
    }
  }
}
